/*
 * main.c source file for extace
 * 
 * /GDK/GNOME sound (esd) system output display program
 * 
 * Based on the original extace written by The Rasterman and Michael Fulbright
 *  
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <config.h>
#include <defaults.h>
#include <globals.h>
#include <protos.h>
#include <math.h>
#include <esd.h>
#include <gtk/gtk.h>

/* See globals.h for variable declarations and DEFINES */

int main(int argc, char **argv)
{

    GtkWidget *vbox;
    GtkWidget *vbox1;
    GtkWidget *main_scope_vbox;
    GtkWidget *hbox;
    GtkWidget *window;
    GtkWidget *button;
    GtkWidget *notebook;
    GtkWidget *frame;
    GtkWidget *sep;
    GtkWidget *label;
    GtkWidget *scale;
    GtkWidget *box;
    GtkWidget *table;
    GtkObject *adj1;
    GtkTooltips *tip;
    GSList *group;

    g_thread_init(NULL);
    gtk_init(&argc, &argv);
    gdk_imlib_init();

    init();		/* initialize all global variables */
    make_extace_dirs();	/* make conf dir if it doesn't exist */
    mem_alloc();	/* Alloate memory for the buffers */
    create_initial_colormaps();	/* Create colormaps if you don't have em */
    read_config();	/* read config file */

    gtk_widget_push_visual(gdk_imlib_get_visual());
    gtk_widget_push_colormap(gdk_imlib_get_colormap());

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    main_win_ptr = window;
    gtk_widget_set_uposition(window, main_x_origin, main_y_origin);
    gtk_window_set_title(GTK_WINDOW(window),"eXtace " VERSION);
    gtk_container_border_width(GTK_CONTAINER(window),0);
    gtk_signal_connect(GTK_OBJECT(window),"destroy_event",
	    GTK_SIGNAL_FUNC(leave),NULL);
    gtk_signal_connect(GTK_OBJECT(window),"delete_event",
	    GTK_SIGNAL_FUNC(leave),NULL);
    gtk_signal_connect (GTK_OBJECT (window), "button_press_event",
	    (GtkSignalFunc) button_notify_event, NULL);
    gtk_signal_connect (GTK_OBJECT (window), "button_release_event",
	    (GtkSignalFunc) button_notify_event, NULL);
    gtk_signal_connect (GTK_OBJECT (window), "motion_notify_event",
	    (GtkSignalFunc) motion_notify_event, NULL);

    gtk_widget_set_events (window, GDK_EXPOSURE_MASK
	    | GDK_BUTTON_PRESS_MASK
	    | GDK_BUTTON_RELEASE_MASK
	    | GDK_POINTER_MOTION_HINT_MASK
	    | GDK_POINTER_MOTION_MASK);
    gtk_window_set_policy(GTK_WINDOW(window), TRUE,	/* allow shrink */
	    TRUE, 	/* allow grow */
	    FALSE);	/* auto shrink */

    tip = gtk_tooltips_new();


    gtk_widget_realize(window);
    init_gc(window);

    vbox=gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(window), vbox);
    gtk_widget_show(vbox);
    hbox=gtk_hbox_new(TRUE,0);

    button=gtk_button_new_with_label("Wire 3D");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_tooltips_set_tip(tip, button, "3D Wireframe FFT", NULL);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(button_3d_fft),(gpointer)WIRE_3D);

    button=gtk_button_new_with_label("Landform 3D");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_tooltips_set_tip(tip, button, "3D Landform FFT", NULL);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(button_3d_fft),(gpointer)FILL_3D);

    button=gtk_button_new_with_label("Graphic EQ");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_tooltips_set_tip(tip, button, "Graphic Equalizer", NULL);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(button_2d_fft),NULL);

    button=gtk_button_new_with_label("Scope");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_tooltips_set_tip(tip, button, "Oscilliscope", NULL);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(button_oscilloscope),NULL);

    button=gtk_button_new_with_label("3D Spikes");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_tooltips_set_tip(tip, button, "3D Hi-resolution FFT", NULL);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(button_3d_detailed),NULL);

    button=gtk_button_new_with_label("Horiz Spectragram");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_tooltips_set_tip(tip, button, "Horizontal Spectragram", NULL);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(button_horiz_specgram),NULL);

    button=gtk_button_new_with_label("Vert Spectragram");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_tooltips_set_tip(tip, button, "Vertical Spectragram", NULL);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(button_vert_specgram),NULL);

    button=gtk_button_new_with_label("About");
    about_but_ptr = button;
    gtk_tooltips_set_tip(tip, button, "About eXtace", NULL);
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(button_about),NULL);

    optionsbut=gtk_toggle_button_new_with_label("Options");
    gtk_tooltips_set_tip(tip, optionsbut, "eXtace Advanced Options", NULL);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(optionsbut), FALSE);
    gtk_box_pack_start(GTK_BOX(hbox),optionsbut,TRUE,TRUE,0);
    gtk_widget_show(optionsbut);
    gtk_signal_connect(GTK_OBJECT(optionsbut),"toggled",
	    GTK_SIGNAL_FUNC(button_options),NULL);

    button=gtk_button_new_with_label("Close");
    gtk_tooltips_set_tip(tip, button, "Close eXtace!!", NULL);
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
	    GTK_SIGNAL_FUNC(leave),NULL);

    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);

    disp=gtk_drawing_area_new();
    gtk_signal_connect( GTK_OBJECT(disp),"configure_event",
	    (GtkSignalFunc)main_disp_configure, NULL);
    gtk_signal_connect( GTK_OBJECT(disp),"expose_event",
	    (GtkSignalFunc)main_disp_expose, NULL);
    gtk_box_pack_start(GTK_BOX(vbox),disp,TRUE,TRUE,0);
    gtk_widget_realize(disp);
    gtk_widget_set_usize(disp,width,height);
    win=disp->window;
    gc=gdk_gc_new(win);
    gdk_gc_copy(gc,disp->style->white_gc);
    pixmap=gdk_pixmap_new(win,width,height,
	    gtk_widget_get_visual(disp)->depth);
    gdk_draw_rectangle( pixmap,
	    disp->style->black_gc,
	    TRUE, 0,0,
	    width,height);
    gdk_window_set_back_pixmap(disp->window,pixmap,0);
    gtk_tooltips_set_tip(tip,disp, "The middle button will get you the color picker", NULL);
    gtk_widget_show(disp);

    /* options window */
    options = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(options),"eXtace Options");
    gtk_signal_connect(GTK_OBJECT(options),"delete_event",
	    GTK_SIGNAL_FUNC(close_options),optionsbut);
    gtk_container_border_width(GTK_CONTAINER(options),2);

    notebook = gtk_notebook_new ();
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
    gtk_container_add(GTK_CONTAINER(options), notebook);
    gtk_widget_show(notebook);


    frame = gtk_frame_new("General Controls");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_widget_show(frame);

    label = gtk_label_new("General Options");
    gtk_notebook_append_page(GTK_NOTEBOOK (notebook), frame, label);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    label = gtk_label_new("Lag factor");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    lag_adj = gtk_adjustment_new(lag,lag_min,(float)(BUFS*nsamp),1.0,100.0,10.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(lag_adj));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
//    GTK_ADJUSTMENT (lag_adj)->value = lag;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
	    GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (lag_adj), "value_changed",
	    GTK_SIGNAL_FUNC (slider_changed), (gpointer)LAG);
    gtk_widget_show(scale);

    label = gtk_label_new("Number of bands (low res displays only)");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new((float)bands,16.0,(float)MAXBANDS+1,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
//    GTK_ADJUSTMENT (adj1)->value = bands;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
	    GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
	    GTK_SIGNAL_FUNC (slider_changed), (gpointer)BANDS);
    gtk_widget_show(scale);

    label = gtk_label_new("Sensitivity");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new((float)sensitivity,0.0,sensitivity_max,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
//    GTK_ADJUSTMENT (adj1)->value = sensitivity;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
	    GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
	    GTK_SIGNAL_FUNC (slider_changed), (gpointer)SENSE);
    gtk_widget_show(scale);

    label = gtk_label_new("Bar Decay Speed");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new((float)bar_decay_speed,1.0,bar_decay_max,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
//    GTK_ADJUSTMENT (adj1)->value = bar_decay_val;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
	    GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
	    GTK_SIGNAL_FUNC (slider_changed), (gpointer)DECAY);
    gtk_widget_show(scale);

    label = gtk_label_new("Peak Hold Time");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new((float)trailer_hold_time,1.0,trailer_hold_max,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
//    GTK_ADJUSTMENT (adj1)->value = trailer_hold_time;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
	    GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
	    GTK_SIGNAL_FUNC (slider_changed), (gpointer)TRAIL_HOLD);
    gtk_widget_show(scale);


    label = gtk_label_new("Peak Decay Speed");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new((float)trailer_decay_speed,1.0,11.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
//    GTK_ADJUSTMENT (adj1)->value = trailer_decay_speed;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
	    GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
	    GTK_SIGNAL_FUNC (slider_changed), (gpointer)TRAIL_DECAY);
    gtk_widget_show(scale);

    sep = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox),sep,TRUE, TRUE, 0);
    gtk_widget_show(sep);

    hbox = gtk_hbox_new(TRUE,0);
    gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,FALSE,0);

    button = gtk_toggle_button_new_with_label("Invert Y-axis");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)FLIP);
    if (flip == -1)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button),1);
    gtk_widget_show(button);

    button = gtk_toggle_button_new_with_label("Leading Edge Shown");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SUPPRESS);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), !show_leader);
    gtk_widget_show(button);

    button = gtk_toggle_button_new_with_label("Bar Decay Disabled");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)DECAY);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), slide);
    gtk_widget_show(button);

    button = gtk_toggle_button_new_with_label("Peak Decay Disabled");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)TRAIL_DECAY);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), trailer_slide);
    gtk_widget_show(button);

    gtk_widget_show(hbox);

    hbox = gtk_hbox_new(TRUE,0);
    gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,FALSE,0);

    button = gtk_radio_button_new_with_label(NULL, "Linear X axis");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)LINEAR);
    if (axis_type == LINEAR)
    {
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
    }
    gtk_widget_show(button);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group, "Logarithmic X axis");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)LOG);
    if (axis_type == LOG)
    {
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
    }
    gtk_widget_show(button);

    button = gtk_toggle_button_new_with_label("Backing Pixmap Disabled");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)BACK_PIXMAP);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), use_back_pixmap);
    gtk_widget_show(button);

    button = gtk_toggle_button_new_with_label("Perspective Tilt Disabled");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)PERS_TILT);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), tilt);

//    gtk_widget_show(hbox);
    gtk_widget_show_all(vbox);


    vbox = gtk_vbox_new(FALSE,0);
    main_scope_vbox = vbox;
    label = gtk_label_new("Scope Options");
    gtk_notebook_append_page(GTK_NOTEBOOK (notebook), vbox, label);
    gtk_widget_show_all(vbox);

    hbox = gtk_hbox_new(FALSE,0);
    gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

    frame = gtk_frame_new("Scope Mode");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(hbox),frame,TRUE,TRUE,0);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    label = gtk_label_new("Scope Mode");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);

    button = gtk_radio_button_new_with_label(NULL, "Gradient Scope");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (scope_mode), (gpointer)GRAD_SCOPE);
    if (scope_sub_mode == GRAD_SCOPE)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group, "Dot Scope");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (scope_mode), (gpointer)DOT_SCOPE);
    if (scope_sub_mode == DOT_SCOPE)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group, "Line Scope");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (scope_mode), (gpointer)LINE_SCOPE);
    if (scope_sub_mode == LINE_SCOPE)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);

    frame = gtk_frame_new("Scope Controls");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(hbox),frame,TRUE,TRUE,0);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    button = gtk_toggle_button_new_with_label("Trace Stabilizer Disabled");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)STABLE);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), stabilized);

    button = gtk_toggle_button_new_with_label("Scope Graticule Disabled");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)GRATICULE);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), show_graticule);

    frame = gtk_frame_new("Sync/Trigger Controls");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(main_scope_vbox),frame,TRUE,TRUE,0);

    hbox = gtk_hbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), hbox);

    button = gtk_radio_button_new_with_label(NULL, "Sync to Left Channel");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SYNC_LEFT);
    if (sync_to_left)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group, "Sync to Right Channel");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SYNC_RIGHT);
    if (sync_to_right)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group, "Sync Independantly");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SYNC_INDEP);
    if (sync_independant)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);

    gtk_widget_show_all(main_scope_vbox);


    hbox = gtk_hbox_new(FALSE,0);

    label = gtk_label_new("FFT Options");
    gtk_notebook_append_page(GTK_NOTEBOOK (notebook), hbox, label);

    frame = gtk_frame_new("Window Functions");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(hbox),frame,TRUE,TRUE,0);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);
    button = gtk_radio_button_new_with_label(NULL,"Hamming");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(window_func == HAMMING)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "pressed",
	    GTK_SIGNAL_FUNC (setup_datawindow), (gpointer)HAMMING);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Hanning");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(window_func == HANNING)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "pressed",
	    GTK_SIGNAL_FUNC (setup_datawindow), (gpointer)HANNING);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Blackman");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(window_func == BLACKMAN)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "pressed",
	    GTK_SIGNAL_FUNC (setup_datawindow), (gpointer)BLACKMAN);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Blackman-Harris");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(window_func == BLACKMAN_HARRIS)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "pressed",
	    GTK_SIGNAL_FUNC (setup_datawindow), (gpointer)BLACKMAN_HARRIS);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Gaussian");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(window_func == GAUSSIAN)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "pressed",
	    GTK_SIGNAL_FUNC (setup_datawindow), (gpointer)GAUSSIAN);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Welch");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(window_func == WELCH)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "pressed",
	    GTK_SIGNAL_FUNC (setup_datawindow), (gpointer)WELCH);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Parzen");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(window_func == PARZEN)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "pressed",
	    GTK_SIGNAL_FUNC (setup_datawindow), (gpointer)PARZEN);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Rectangular");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(window_func == RECTANGULAR)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "pressed",
	    GTK_SIGNAL_FUNC (setup_datawindow), (gpointer)RECTANGULAR);

    vbox1 = gtk_vbox_new(FALSE,0);
    gtk_box_pack_start(GTK_BOX(hbox),vbox1,TRUE,TRUE,0);

    frame = gtk_frame_new("Window Function Options");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(vbox1),frame,TRUE,TRUE,0);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    label = gtk_label_new("Window Function Active Width");

    button = gtk_radio_button_new_with_label(NULL,"Full Width");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(winstyle == FULL)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)FULL);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Half Width");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(winstyle == HALF)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)HALF);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Quarter Width");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(winstyle == QUARTER)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)QUARTER);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Eighth Width");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(winstyle == EIGHTH)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)EIGHTH);




    frame = gtk_frame_new("FFT Controls");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(vbox1),frame,TRUE,TRUE,0);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    label = gtk_label_new("FFT Audio Signal Source");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);

    button = gtk_radio_button_new_with_label(NULL,"Left Channel");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(fft_signal_source == LEFT)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)LEFT);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Right Channel");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(fft_signal_source == RIGHT)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)RIGHT);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"Both Channels");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(fft_signal_source == COMPOSITE)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)COMPOSITE);


    frame = gtk_frame_new("FFT Width");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(hbox),frame,TRUE,TRUE,0);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    label = gtk_label_new("FFT Width in samples");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);

//    button = gtk_radio_button_new_with_label(NULL,"512");
//    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
//    if(nsamp == 512)
//	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
//    gtk_signal_connect (GTK_OBJECT (button), "toggled",
//	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SAM_512);
//
//    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(NULL,"1024");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(nsamp == 1024)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SAM_1024);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"2048");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(nsamp == 2048)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SAM_2048);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"4096");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(nsamp == 4096)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SAM_4096);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"8192");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(nsamp == 8192)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SAM_8192);

    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(group,"16384");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if(nsamp == 16384)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
	    GTK_SIGNAL_FUNC (button_handle), (gpointer)SAM_16384);

    gtk_widget_show_all(hbox);

    box = gtk_vbox_new(TRUE,0);
    label = gtk_label_new("Misc Options");
    gtk_notebook_append_page(GTK_NOTEBOOK (notebook), box, label);

    frame = gtk_frame_new("Spectragram Controls");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(box), frame,TRUE,TRUE,0);

    vbox = gtk_vbox_new(TRUE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    label = gtk_label_new("Main window refresh rate (fps)");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new((float)refresh_rate,1.0,refresh_max,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
//    GTK_ADJUSTMENT (adj1)->value = refresh_rate;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
	    GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
	    GTK_SIGNAL_FUNC (slider_changed), (gpointer)REFRESH_RATE);

    label = gtk_label_new("Spectragram Scroll Speed");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new((float)tape_scroll,1.0,tape_scroll_max,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
//    GTK_ADJUSTMENT (adj1)->value = tape_scroll;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
	    GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
	    GTK_SIGNAL_FUNC (slider_changed), (gpointer)TAPE_SCROLL);


    frame = gtk_frame_new("Sound Source");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start(GTK_BOX(box), frame,TRUE,TRUE,0);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    label = gtk_label_new("Choose ALSA Device and SubDevice");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);

    button = gtk_radio_button_new_with_label(NULL, "Use Esound");
    gtk_box_pack_start(GTK_BOX(vbox),button,TRUE,TRUE,0);
    if (sound_source == ESD)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
    gtk_signal_connect(GTK_OBJECT(button),"toggled",
	    GTK_SIGNAL_FUNC(button_handle),(gpointer)ESD);

    table = gtk_table_new(2,6,FALSE);
    gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,0);

    label = gtk_label_new("Card");
    gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);
    label = gtk_label_new("Device");
    gtk_table_attach_defaults(GTK_TABLE(table),label,2,3,0,1);
    label = gtk_label_new("Subdevice");
    gtk_table_attach_defaults(GTK_TABLE(table),label,3,4,0,1);

    group = gtk_radio_button_group(GTK_RADIO_BUTTON(button));
    button = gtk_radio_button_new_with_label(group, "Use ALSA Loopback");
    if (sound_source == ALSA)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
    gtk_signal_connect(GTK_OBJECT(button),"toggled",
	    GTK_SIGNAL_FUNC(button_handle),(gpointer)ALSA);
    gtk_table_attach_defaults(GTK_TABLE(table),button,0,1,1,2);
#ifndef HAVE_ALSA
    gtk_widget_set_sensitive(button,FALSE);
#endif

    adj1 = gtk_adjustment_new(alsa_card, 0, 8, 1,7,8);
    button = gtk_spin_button_new(GTK_ADJUSTMENT(adj1),0.0,0);
    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(button),TRUE);
    gtk_signal_connect(GTK_OBJECT(adj1), "value_changed",
	    GTK_SIGNAL_FUNC(alsa_adjust), (gpointer)ALSA_CARD);
    gtk_table_attach_defaults(GTK_TABLE(table),button,1,2,1,2);
#ifndef HAVE_ALSA
    gtk_widget_set_sensitive(button,FALSE);
#endif

    adj1 = gtk_adjustment_new(alsa_device, 0, 8, 1,7,8);
    button = gtk_spin_button_new(GTK_ADJUSTMENT(adj1),0.0,0);
    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(button),TRUE);
    gtk_signal_connect(GTK_OBJECT(adj1), "value_changed",
	    GTK_SIGNAL_FUNC(alsa_adjust), (gpointer)ALSA_DEVICE);
    gtk_table_attach_defaults(GTK_TABLE(table),button,2,3,1,2);
#ifndef HAVE_ALSA
    gtk_widget_set_sensitive(button,FALSE);
#endif

    adj1 = gtk_adjustment_new(alsa_sub_dev, 0, 8, 1,7,8);
    button = gtk_spin_button_new(GTK_ADJUSTMENT(adj1),0.0,0);
    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(button),TRUE);
    gtk_signal_connect(GTK_OBJECT(adj1), "value_changed",
	    GTK_SIGNAL_FUNC(alsa_adjust), (gpointer)ALSA_SUB_DEV);
    gtk_table_attach_defaults(GTK_TABLE(table),button,3,4,1,2);
#ifndef HAVE_ALSA
    gtk_widget_set_sensitive(button,FALSE);
#endif

    gtk_widget_show_all(box);


    dir_win = gtk_window_new(GTK_WINDOW_DIALOG);
    dir_win_ptr = dir_win;
    gtk_widget_set_uposition(dir_win, dir_x_origin, dir_y_origin);
    gtk_window_set_title(GTK_WINDOW(dir_win),"Direction");
    gtk_container_border_width(GTK_CONTAINER(dir_win),2);
    gtk_widget_set_usize(dir_win,dir_width,dir_height);
    gtk_signal_connect(GTK_OBJECT(dir_win),"delete_event",
	    GTK_SIGNAL_FUNC(close_dir_win),NULL);
    gtk_signal_connect (GTK_OBJECT (dir_win), "motion_notify_event",
	    (GtkSignalFunc) dir_motion, NULL);
    gtk_signal_connect (GTK_OBJECT (dir_win), "focus_out_event",
	    (GtkSignalFunc) dir_save_state, NULL);



    gtk_widget_set_events (dir_win, GDK_EXPOSURE_MASK
	    | GDK_BUTTON_PRESS_MASK
	    | GDK_BUTTON_RELEASE_MASK
	    | GDK_FOCUS_CHANGE_MASK
	    | GDK_POINTER_MOTION_HINT_MASK
	    | GDK_POINTER_MOTION_MASK);

    dir_area = gtk_drawing_area_new();
    gtk_tooltips_set_tip(tip, dir_area, "Drag the pointer ", NULL);
    gtk_drawing_area_size (GTK_DRAWING_AREA (dir_area), 
	    dir_win->allocation.width, 
	    dir_win->allocation.height);
    gtk_signal_connect( GTK_OBJECT(dir_area),"configure_event",
	    (GtkSignalFunc)dir_configure, NULL);
    gtk_signal_connect( GTK_OBJECT(dir_area),"expose_event",
	    (GtkSignalFunc)dir_expose, NULL);

    gtk_container_add(GTK_CONTAINER(dir_win), dir_area);

    gtk_widget_show(dir_area);
    if (dir_win_present)
    {
	gtk_widget_show(dir_win);
	update_pointer();
    }

    gtk_window_set_policy(GTK_WINDOW(dir_win), FALSE,	/* allow shrink */
	    TRUE, 	/* allow grow */
	    FALSE);	/* auto shrink */

    gtk_window_set_policy(GTK_WINDOW(options), FALSE,	/* allow shrink */
	    TRUE, 	/* allow grow */
	    FALSE);	/* auto shrink */

    setup_datawindow(NULL,(WindowFunction)window_func);
    init_colortab(0);

    gtk_widget_show_all(window);
    gtk_tooltips_enable(tip);
    ready = 1;		/* All set */

    if (mode == STARS)/* gotta emit it by hand due to config file */
	gtk_signal_emit_by_name(GTK_OBJECT(about_but_ptr),"clicked");
    else
	draw_start();

    open_sound();
    audio_thread_starter();
    gdk_threads_enter();
    gtk_main();
    gdk_threads_leave();
    return 0;
}
