/*
 * sound.c extace source file
 * 
 * /GDK/GNOME sound (esd) system output display program
 * 
 * Copyright (C) 1999 by Dave J. Andruczyk 
 * 
 * Based on the original extace written by The Rasterman and Michael Fulbright
 *  
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <config.h>
#include <globals.h>
#include <protos.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include <esd.h>
#include <asm/errno.h>
#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#ifdef HAVE_ALSA
#include <sys/asoundlib.h>
#endif

/*--- globals to this file */
#ifdef HAVE_ALSA
snd_pcm_loopback_t *alsa_handle;
snd_pcm_format_t *format;
#endif
GtkWidget *errbox;
int errorbox_up;
/* Experimental variables to get threading to behave... */

static int read_started = 0;
pthread_t esound_thread;
gint esd_handle = 0;
int esd_locked = 0;
int bytes_2_move = 0;
int bytes_moved = 0;

#ifdef HAVE_ALSA
int alsa_locked = 0;
pthread_t alsa_thread;
#endif

/*--- globals to this file */


int open_sound(void)
{
    int handle = -1;
#ifdef HAVE_ALSA
    int err = 0;
    format = malloc(sizeof(snd_pcm_format_t));
    memset((void *)format, 0, sizeof(snd_pcm_format_t));
#endif


    switch (sound_source)
    {
	case ESD:
	    esd_handle=esd_monitor_stream(ESD_BITS16|ESD_STEREO|ESD_STREAM|ESD_MONITOR,RATE,NULL,"extace");
	    if (esd_handle)
	    {
		handle = 1;
		read_started = 0;
	    }

	    break;


#ifdef HAVE_ALSA
	case ALSA:
#ifdef ALSA_DEBUG
	    printf("opening alsa device \n");
#endif
	    err = snd_pcm_loopback_open(&alsa_handle, alsa_card, alsa_device, alsa_sub_dev, SND_PCM_LB_OPEN_PLAYBACK);
	    if (err <0)
	    {
		printf("open failed: %s\n", snd_strerror(err)); 
	    }
	    else
	    {
		handle = 1;
#ifdef ALSA_DEBUG
		printf("ALSA loopback device opened successfully\n");
#endif
		snd_pcm_loopback_block_mode(alsa_handle, 1);
		format->format = SND_PCM_SFMT_S16_LE;
		format->rate = RATE;
		format->voices = 2;
		err = snd_pcm_loopback_format(alsa_handle, format);
		if (err <0)
		{
		    printf("format info failed: %s\n", snd_strerror(err)); 
		}
#ifdef ALSA_DEBUG
		else
		{
		    printf("ALSA format setup worked OK\n");
		    printf("rate %i, channels %i\n",format->rate, format->voices);
		}
#endif
	    }
	    g_free(format);
	    break;
#endif

    }
    if (handle < 0)
    {
	GtkWidget *label;
	if (errorbox_up)
	    return(-1); /* ERROR window already onscreen */
	errbox = gtk_window_new(GTK_WINDOW_DIALOG);
	
	gtk_window_set_title(GTK_WINDOW(errbox),"ERROR!!!");
	label = gtk_label_new("Error, Cannot connect to Sound source!!\n. PLease make sure you havethe proper setting in the options panel.\n");
	gtk_container_add(GTK_CONTAINER(errbox), label);
	gtk_widget_show_all(errbox);

	gtk_signal_connect(GTK_OBJECT(errbox), "delete_event",
		GTK_SIGNAL_FUNC(error_close_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(errbox), "destroy_event",
		GTK_SIGNAL_FUNC(error_close_cb), NULL);
	errorbox_up = 1;
	return(-1);
    }
    else
    {
	plan = rfftw_create_plan(nsamp, FFTW_FORWARD, FFTW_ESTIMATE);
	return(0);
    }
}
void close_sound(void)
{
    switch(sound_source)
    {
	case ESD:
/*	    printf("closing esd_handle\n");  */
	    esd_close(esd_handle);
	    break;
#ifdef HAVE_ALSA
	case ALSA:
/*	    printf("closing alsa_handle\n");  */
	    snd_pcm_loopback_close(alsa_handle);
	    break;
#endif
	default:
	    break;
    }
}
        
int audio_thread_stopper()
{
    int retcode = 0;
    if (read_started)
    {
	switch (sound_source)
	{
	    case ESD:
		retcode = pthread_cancel(esound_thread);
		if (retcode != 0)
		    printf("Error attempting cancel esd thread\n");
		else 
/*		    printf("ESD thread stopped successfully\n");  */
		esd_locked = 0;
		break;
#ifdef HAVE_ALSA
	    case ALSA:
		retcode = pthread_cancel(alsa_thread);
		if (retcode != 0)
		    printf("Error attempting cancel main ALSA thread\n");
		else 
/*		    printf("ALSA main thread stopped successfully\n");  */
		alsa_locked = 0;
		break;
#endif
		default:
		break;
	}
    }
    read_started = 0;
    return 0;
}

int audio_thread_starter()
{
    int retcode = 0;
    if (read_started)
	printf("Error, reader already running!!\n");
    else
    {
	switch (sound_source)
	{
	    case ESD:
		/* since we want to audio to NOT be read in the main gtk loop
		 * we'll use the gtk input function as a helper. IT will awaken
		 * our esound reader thread whenever data is ready. Save us 
		 * from having to setup a poll loop by hand, as gdk/gtk does
		 * it pretty well already */
		retcode = pthread_create(&esound_thread,
			NULL, /*Thread attributes */
			esd_starter_thread,
			NULL /* args passed to thread */);
		if (retcode != 0)
		    printf("Error attempting to create Esound thread\n");
		break;
#ifdef HAVE_ALSA
	    case ALSA:
		retcode = pthread_create(&alsa_thread,
			NULL,
			alsa_starter_thread,
			alsa_handle);
		if (retcode != 0)
		    printf("Error attempting to create MAIN ALSA thread\n");
		break;
#endif

	    default:
		break;

	}
    }
    read_started = 1;
    return(0);

}
    
#ifdef HAVE_ALSA
void alsa_reader_thread(void *private_data, char *buffer, size_t count)
{
/*    printf("audio has arrived, locking mutex (ALSA_HELPER)\n"); */
//    pthread_mutex_lock(alsa_mutex);
/*    printf("mutex locked (ALSA_HELPER)\n"); */
//    while(alsa_locked == 1)
//    {	/* loop, waiting on other thread to go to sleep */
/*	printf("looping on condition variable  (ALSA_HELPER)\n"); */
//	pthread_cond_wait(alsa_condition, alsa_mutex);
//    }
/*    printf("passed condition test (ALSA_HELPER)\n"); */

    /* set predicate to block other thread */
    alsa_locked = 1;
    bytes_2_move = count;
    bytes_moved = 0;
    /* ring_pos is in ELEMENTS, NOT BYTES!!!!, since the buffer is of shorts
     * incrementing ring_pos by 1 actually moves 2 bytes ahead */

    if (ring_pos+(bytes_2_move/2) > ring_end) 
    {
	/* fill up to end of ring buffer, but don't jump boundary */
	memcpy(audio_ring + ring_pos,
		buffer,
		(ring_end - ring_pos)*2); /*need bytes, not elements */
	bytes_moved = (ring_end - ring_pos)*2;

	/* wrap to beginning */
	memcpy(audio_ring,
		buffer + bytes_moved,
		bytes_2_move - bytes_moved);
	/* mark where we are .. */
	ring_pos = (bytes_2_move - bytes_moved)/2; /* need elements NOT bytes*/
    }         
    else
    {
	memcpy(audio_ring + ring_pos,
		buffer,
		bytes_2_move);
	ring_pos += bytes_2_move/2; /* mark where we are in ringbuffer */
    } 


    /* unset predicate */
    alsa_locked = 0;
    /* wakeup other thread */
/*    printf("sending wakeup signal (ALSA_HELPER)\n"); */
//    pthread_cond_broadcast(alsa_condition);
    /* unlock mutex so other thread can run */
/*    printf("unlocking mutex (ALSA_HELPER)\n"); */
//    pthread_mutex_unlock(alsa_mutex);
}
#endif
void esd_reader_thread(gpointer data, gint source, GdkInputCondition condition)
{
    int count = 0;
/*    printf("audio has arrived, locking mutex (ESD_HELPER)\n"); */
//    pthread_mutex_lock(esd_mutex);
/*    printf("mutex locked (ESD_HELPER)\n"); */
//    while(esd_locked == 1)
//    {	/* loop, waiting on other thread to go to sleep */
/*	printf("looping on condition variable  (ESD_HELPER)\n"); */
//	pthread_cond_wait(esd_condition, esd_mutex);
//    }
/*    printf("passed condition test (ESD_HELPER)\n"); */

    /* set predicate to block other thread */
    esd_locked = 1;
    bytes_moved = 0;
/*    printf("esd_reader running\n"); */

    /* copy fd so that reader thread can copy data to ringbuffer */
    count = read(source, incoming_buf, to_get); 
    if (count > 0)
    {
	bytes_2_move = count;
	if (ring_pos+(bytes_2_move/2) > ring_end)
	{
	    /* fill up to end of ring buffer, but don't jump boundary */
	    memcpy(audio_ring+ring_pos,
		    incoming_buf,
		    (ring_end - ring_pos)*2); /* bytes NOT elements */
	    bytes_moved = (ring_end - ring_pos)*2;
	    /* wrap to beginning */
	    memcpy(audio_ring,
		    incoming_buf + bytes_moved,
		    bytes_2_move - bytes_moved);
	    /* mark where we are .. */
	    ring_pos =  (bytes_2_move - bytes_moved)/2; /* need elements not BYTES */

	}
	else
	{
	    memcpy(audio_ring+ring_pos,
		    incoming_buf,
		    bytes_2_move);
	    ring_pos += bytes_2_move/2; /*mark where we are in ringbuffer*/
	}                  
    }

    /* unset predicate */
/*    printf("esd_reader exiting count = %i, esd_data_count = %i\n",count,esd_data_count); */
    esd_locked = 0;
    /* wakeup other thread */
/*    printf("sending wakeup signal (ESD_HELPER)\n"); */
//    pthread_cond_broadcast(esd_condition);
    /* unlock mutex so other thread can run */
/*    printf("unlocking mutex (ESD_HELPER)\n"); */
//    pthread_mutex_unlock(esd_mutex);
}

void *esd_starter_thread(void * params)
{
    /* Creates the GTK input handler for Esound, then evaporates */
    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);

    /*	fcntl(esd_handle, F_SETFL, O_NONBLOCK); */
    tag = gdk_input_add(esd_handle, 
	    GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
	    esd_reader_thread, NULL);
    return(0);
}

#ifdef HAVE_ALSA

void *alsa_starter_thread(void * handle)
{
    /* Creates the ALSA callback handler for ALSA, then sleeps indefinitely */
    int retcode=0;
    pthread_t callback_thread;
    snd_pcm_loopback_callbacks_t callbacks;

    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
    memset(&callbacks, 0, sizeof(callbacks));
    callbacks.data = alsa_reader_thread;
    callbacks.position_change = loopback_position_change;
    callbacks.silence = loopback_silence;
    callbacks.format_change = loopback_format_change;
    callbacks.max_buffer_size = callback_buffer_size;

    snd_pcm_loopback_read(handle, &callbacks);
    if (keep_reading == 1)
    {
/*    printf("loopback died for some stupid reason, restarting\n"); */
    retcode = pthread_create(&callback_thread, NULL, alsa_starter_thread,alsa_handle);
    }
    else
    {
/*	#printf("alsa shutting down, don't restart callback thread \n"); */
    }
    return(0);
}

void loopback_position_change(void *private_data,unsigned int pos) 
{
    printf("Alsa POSITION change callback called, doing nothing\n");
}

void loopback_format_change(void *private_data,snd_pcm_format_t *format) 
{
    printf("Alsa FORMAT change callback called, doing nothing\n");
}

void loopback_silence(void *private_data,size_t count) 
{
    printf("Alsa silence callback called, doing nothing\n");
}
#endif

void error_close_cb(GtkWidget *widget, gpointer *data)
{
      printf("Cannot connect to sound source, check options.\n");
	gtk_widget_destroy(errbox);
}

void alsa_adjust(GtkWidget *widget, gpointer *data)
{
    int retval=0;
    switch ((gint)data)
    {
	case ALSA_CARD:
	    alsa_card = GTK_ADJUSTMENT(widget)->value;
	    break;
	case ALSA_DEVICE:
	    alsa_device = GTK_ADJUSTMENT(widget)->value;
	    break;
	case ALSA_SUB_DEV:
	    alsa_sub_dev = GTK_ADJUSTMENT(widget)->value;
	    break;
	default:
	    break;
    }
    if (sound_source == ALSA)
    {
	keep_reading = 0;
	audio_thread_stopper();
	usleep(2000);
	close_sound();
	retval=open_sound();
	keep_reading = 1;
	if (retval >= 0)
	    audio_thread_starter();
    }
}

